var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useEffect, useRef, useState } from "react";
import TextNodeSearchInputDropdown from "./TextNodeSearchInputDropdown";
import { configuration } from "../../../configuration/configure";
import useInputElementValue from "./useInputElementValue";
import useKeyboardNavigation from "./useKeyboardNavigation";
const RE_BLUR_ON_FOCUS_THRESHOLD_MILLiSECONDS = 100;
function TextNodeSearchInput({ placeholder, isAutoFocus, textNodeTree: { currentNode, errorMessage, goToNode, isDestinationFound, isSettingOptions, options, resetOptionsToCurrentNodeChildren, retry, searchForNodes } }) {
    const inputElementRef = useRef(null);
    const isDelete = useRef(false);
    const isBlurredWithErrorRef = useRef(false);
    const blurTimestampRef = useRef(-1);
    const [isInFocusTracker, setIsInFocusTracker] = useState(false);
    const { inputElementValue, setInputByNode, setInputByValue, } = useInputElementValue(inputElementRef);
    const keyboardNavigation = useKeyboardNavigation(options.length, goToNode, (index) => {
        setInputByNode({ node: options[index], isKeyboardNavigation: true });
    });
    useEffect(() => {
        var _a;
        keyboardNavigation.stop();
        setInputByNode({ node: currentNode });
        if (currentNode === null || currentNode === void 0 ? void 0 : currentNode.isLeaf) {
            (_a = inputElementRef.current) === null || _a === void 0 ? void 0 : _a.blur();
        }
    }, [currentNode]);
    useEffect(() => {
        if (!keyboardNavigation.isNavigating) {
            setInputByNode({ node: currentNode });
        }
    }, [keyboardNavigation.isNavigating]);
    useEffect(() => {
        (() => __awaiter(this, void 0, void 0, function* () {
            const isAutomaticChoice = options.length === 1 && options[0].isLeaf && !isDelete.current;
            if (isAutomaticChoice) {
                yield goToNode(0);
            }
            isDelete.current = false;
        }))();
    }, [options]);
    const handleValueChanged = (value, caretIndexInValue) => __awaiter(this, void 0, void 0, function* () {
        isDelete.current = value.length < inputElementValue.length;
        keyboardNavigation.stop();
        const match = options.find((n) => n.isMatch(value));
        if (match && !isDelete.current) {
            // Case correction
            setInputByNode({ node: match });
        }
        else {
            setInputByValue(value, caretIndexInValue);
        }
        yield searchForNodes(value, caretIndexInValue);
    });
    function handleInputFocus() {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c;
            const blurTimestampDelta = Date.now() - blurTimestampRef.current;
            if (blurTimestampDelta < RE_BLUR_ON_FOCUS_THRESHOLD_MILLiSECONDS) {
                // Rejecting Mobile autofocus after blur
                (_a = inputElementRef.current) === null || _a === void 0 ? void 0 : _a.blur();
                return;
            }
            setIsInFocusTracker(true);
            if (isBlurredWithErrorRef.current) {
                isBlurredWithErrorRef.current = false;
                if (isDestinationFound) {
                    return;
                }
                yield retry(inputElementValue, (_c = (_b = inputElementRef.current) === null || _b === void 0 ? void 0 : _b.selectionStart) !== null && _c !== void 0 ? _c : inputElementValue.length);
            }
            if (keyboardNavigation.isNavigating) {
                setInputByNode({ node: options[keyboardNavigation.optionsIndex], isKeyboardNavigation: true });
                return;
            }
            setInputByNode({ node: currentNode });
            yield resetOptionsToCurrentNodeChildren();
        });
    }
    function handleInputBlur() {
        return __awaiter(this, void 0, void 0, function* () {
            setIsInFocusTracker(false);
            blurTimestampRef.current = Date.now();
            if (errorMessage) {
                isBlurredWithErrorRef.current = true;
            }
            setInputByNode({ node: currentNode });
        });
    }
    const isDroppedDown = !errorMessage && (isSettingOptions || isInFocusTracker && options.length > 0);
    const STYLING = configuration.styling.textNodeSearchInput;
    const getInputElementClassNames = () => {
        let inputElementClassNames = STYLING.inputElementClassNames;
        if (isDestinationFound) {
            inputElementClassNames = inputElementClassNames.concat(STYLING.inputElementValidClassNames);
        }
        return inputElementClassNames.join(" ");
    };
    return (_jsxs("div", { className: STYLING.divElementClassNames.join(" "), "aria-live": "polite", role: "listbox", children: [_jsx("input", { ref: inputElementRef, autoFocus: isAutoFocus, autoCorrect: "off", autoCapitalize: "off", spellCheck: false, type: "text", className: getInputElementClassNames(), value: errorMessage !== null && errorMessage !== void 0 ? errorMessage : inputElementValue, placeholder: placeholder, onKeyDown: (e) => __awaiter(this, void 0, void 0, function* () {
                    yield keyboardNavigation.handleKeyDown(e);
                }), onChange: (e) => __awaiter(this, void 0, void 0, function* () {
                    var _a;
                    yield handleValueChanged(e.target.value, (_a = e.target.selectionStart) !== null && _a !== void 0 ? _a : e.target.value.length);
                }), onBlur: handleInputBlur, onFocus: handleInputFocus, disabled: !!errorMessage && !isDestinationFound, "aria-placeholder": placeholder, "aria-label": configuration.ariaLabel, "aria-expanded": isDroppedDown }), isDroppedDown && _jsx(TextNodeSearchInputDropdown, { options: options.map((n) => n), activeLiElementIndex: keyboardNavigation.optionsIndex, handleChoice: goToNode, isLoading: isSettingOptions })] }));
}
export default TextNodeSearchInput;
