import { useLayoutEffect, useRef, useState } from "react";
function useInputElementValue(inputElementRef) {
    const [inputElementValue, setInputElementValue] = useState("");
    const pendingCaretIndexRef = useRef(null);
    // The element must exist before the caret can be set.
    useLayoutEffect(() => {
        if (!inputElementRef.current || !pendingCaretIndexRef.current) {
            return;
        }
        // Supporting mobile
        setTimeout(() => {
            if (!inputElementRef.current || !pendingCaretIndexRef.current) {
                return;
            }
            inputElementRef.current.selectionStart = pendingCaretIndexRef.current;
            inputElementRef.current.selectionEnd = pendingCaretIndexRef.current;
        }, 0);
    }, [inputElementValue]);
    function setInputByNode({ node, isKeyboardNavigation = false }) {
        if (!node) {
            return;
        }
        if (_isInputElementInFocus() && !isKeyboardNavigation) {
            setInputCaretText(node.query);
        }
        else {
            setInputCaretText(node.presentable);
        }
    }
    function setInputCaretText(caretText) {
        setInputByValue(caretText.value, caretText.caretIndex);
    }
    function setInputByValue(value, caretIndex) {
        setInputElementValue(value);
        pendingCaretIndexRef.current = caretIndex;
    }
    return {
        inputElementValue,
        setInputByNode,
        setInputByValue
    };
    function _isInputElementInFocus() {
        return (document === null || document === void 0 ? void 0 : document.activeElement) === inputElementRef.current;
    }
}
export default useInputElementValue;
