var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useEffect, useRef, useState } from "react";
import { map } from "../../shared/Result";
import { configuration } from "../../configuration/configure";
const ERROR_TIMEOUT_SECONDS = 3;
export function useTextNodeTree(service, destinationResultObserver) {
    const [options, setOptions] = useState([]);
    const [currentNode, setCurrentNode] = useState(null);
    const [destinationResult, setDestinationResult] = useState(null);
    const [errorMessage, setErrorMessage] = useState(null);
    const [isSettingOptions, setIsSettingOptions] = useState(false);
    const setResultIdRef = useRef(-1);
    useEffect(() => {
        destinationResultObserver(destinationResult);
    }, [destinationResult]);
    function searchForNodes(value, caretIndexInValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (value.length < 1) {
                _reset();
                return;
            }
            const promise = service.getNodesByCaretText(value, caretIndexInValue);
            yield _setResult(promise);
        });
    }
    function goToNode(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const node = options[index];
            if (!node) {
                return;
            }
            setCurrentNode(node);
            setDestinationResult(node.result);
            const promise = node.getChildren();
            yield _setResult(promise);
        });
    }
    function retry(value, caretIndexInValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (destinationResult) {
                return;
            }
            let promise = null;
            if (currentNode) {
                promise = currentNode.getChildren();
            }
            else {
                promise = service.getNodesByCaretText(value, caretIndexInValue);
            }
            yield _setResult(promise);
        });
    }
    function resetOptionsToCurrentNodeChildren() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!currentNode) {
                return;
            }
            const result = currentNode.getChildren();
            yield _setResult(result);
        });
    }
    return {
        options,
        currentNode,
        isSettingOptions,
        isDestinationFound: !!destinationResult,
        searchForNodes,
        goToNode,
        resetOptionsToCurrentNodeChildren,
        errorMessage,
        retry
    };
    function _setResult(resultPromise) {
        return __awaiter(this, void 0, void 0, function* () {
            setResultIdRef.current = setTimeout(() => {
                setIsSettingOptions(true);
            }, configuration.apiRequestLoadingIndicatorDebounceMilliseconds);
            const result = yield resultPromise;
            map(result, (s) => {
                setOptions(s);
                clearTimeout(setResultIdRef.current);
                setIsSettingOptions(false);
            }, (f) => {
                console.error(f);
                setErrorMessage(configuration.errorMessageOnApiClientFailure);
                setTimeout(() => {
                    setErrorMessage(null);
                }, ERROR_TIMEOUT_SECONDS * 1000);
                clearTimeout(setResultIdRef.current);
                setIsSettingOptions(false);
            }, (c) => {
                console.debug(c.reason);
            });
        });
    }
    function _reset() {
        setOptions([]);
        setCurrentNode(null);
        setDestinationResult(null);
        setErrorMessage(null);
        setResultIdRef.current = -1;
    }
}
