import { AddressType } from "../core/address/AddressType";
import { configuration } from "../configuration/configure";
export function createToReqDtoMapper() {
    const _zipCodesSelection = configuration.zipCodesSelection;
    const _municipalityCodesSelection = configuration.municipalityCodesSelection;
    function mapNodeToReqDto(node) {
        return {
            value: node.query.value,
            caretIndexInValue: node.query.caretIndex,
            type: _mapAddressTypeToDto(AddressType.Address),
            scope: _createAutocompleteAddressRequestScopeFromAddress(node.result)
        };
    }
    function mapCaretTextToReqDto(value, caretIndexInValue) {
        return {
            value: value,
            caretIndexInValue: caretIndexInValue,
            scope: {
                zipCodes: _zipCodesSelection,
                municipalityCodes: _municipalityCodesSelection
            }
        };
    }
    return {
        mapCaretTextToReqDto,
        mapNodeToReqDto
    };
    function _createAutocompleteAddressRequestScopeFromAddress(address) {
        let leastSpecificity;
        let entranceAddressId;
        switch (address === null || address === void 0 ? void 0 : address.type) {
            case undefined:
                leastSpecificity = _mapAddressTypeToDto(AddressType.Entrance);
                break;
            case AddressType.Entrance:
                entranceAddressId = address.id;
                break;
            case AddressType.Address:
                entranceAddressId = address.entranceAddressId;
                break;
        }
        return {
            leastSpecificity: leastSpecificity,
            entranceAddressId: entranceAddressId,
            zipCodes: _zipCodesSelection,
            municipalityCodes: _municipalityCodesSelection
        };
    }
    function _mapAddressTypeToDto(type) {
        switch (type) {
            case AddressType.Entrance:
                return "adgangsadresse";
            case AddressType.Address:
                return "adresse";
        }
    }
}
