var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { configuration } from "../../configuration/configure";
import { cancelStaleRequest, fail, succeed } from "../../shared/Result";
export function createAutocompleteAddressApiClient() {
    const CONFIGURATION = configuration.autocompleteAddressApiClient;
    const MAX_RESULT_COUNT = CONFIGURATION.maxResultCount.toString();
    function readAutocompleteAddresses(reqDto, abortController) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = _createAutocompleteUrl(reqDto);
            try {
                const response = yield fetch(url, { signal: abortController === null || abortController === void 0 ? void 0 : abortController.signal });
                if (!response.ok) {
                    return fail(new Error(`HTTP ${response.status}: ${response.statusText}.`));
                }
                try {
                    const resDtos = yield response.json();
                    return succeed(resDtos);
                }
                catch (e) {
                    return fail(new Error(`Parsing the response as DTOs failed: ${e}.`));
                }
            }
            catch (e) {
                if (abortController === null || abortController === void 0 ? void 0 : abortController.signal.aborted) {
                    return cancelStaleRequest();
                }
                return fail(new Error(`Fetch failed: ${e}.`));
            }
        });
    }
    return {
        readAutocompleteAddresses
    };
    function _createAutocompleteUrl(reqDto) {
        const searchParameters = new URLSearchParams({
            "q": reqDto.value,
            "caretpos": reqDto.caretIndexInValue.toString(),
            "fuzzy": "", // Not documented but always provided as such in DAWA's own client
            "per_side": MAX_RESULT_COUNT
        });
        if (reqDto === null || reqDto === void 0 ? void 0 : reqDto.type)
            searchParameters.append("type", reqDto.type);
        _appendScope(searchParameters, reqDto.scope);
        return new URL(`${CONFIGURATION.uri}?${searchParameters.toString()}`, CONFIGURATION.host);
    }
    function _appendScope(searchParameters, scopeDto) {
        if (scopeDto) {
            if (scopeDto.entranceAddressId)
                searchParameters.append("adgangsaddresseid", scopeDto.entranceAddressId);
            if (scopeDto.leastSpecificity)
                searchParameters.append("startfra", scopeDto.leastSpecificity);
            if (scopeDto.zipCodes.length > 0)
                searchParameters.append("postnr", scopeDto.zipCodes.join("|"));
            if (scopeDto.municipalityCodes.length > 0)
                searchParameters.append("kommunekode", scopeDto.municipalityCodes.join("|"));
        }
        return searchParameters;
    }
}
