import { createCaretText } from "../core/text-node-search/create-caret-text";
import { createResultingCaretTextQueryNode } from "../core/text-node-search/create-resulting-caret-text-query-node";
import { createAddress } from "../core/address/create-address";
import { AddressType } from "../core/address/AddressType";
export function createFromResDtoMapper() {
    function mapResDtoToNode(resDto, getNodesByNode) {
        const isLeaf = resDto.type === "adresse";
        const isRoot = resDto.type === "vejnavn";
        const address = _mapResDtoToAddress(resDto);
        const query = createCaretText(resDto.tekst, resDto.caretpos);
        const presentable = createCaretText(resDto.forslagstekst);
        return createResultingCaretTextQueryNode(isRoot, isLeaf, getNodesByNode, query, presentable, address);
    }
    return {
        mapResDtoToNode
    };
    function _mapResDtoToAddress(dto) {
        if (!_isAddress(dto)) {
            return null;
        }
        return createAddress(dto.data.id, _mapToAddressType(dto.type), dto.data.vejnavn, dto.data.husnr, dto.data.postnr, dto.data.postnrnavn, dto.data.etage, dto.data.dør, dto.data.vejkode, dto.data.adresseringsvejnavn, dto.data.supplerendebynavn, dto.data.kommunekode, dto.data.status, dto.data.darstatus, dto.data.adgangsadresseid, dto.data.y, dto.data.x, dto.data.href);
    }
    function _isAddress(dto) {
        return !!dto.data.id &&
            !!dto.type &&
            dto.type !== "vejnavn" &&
            !!dto.data.vejnavn &&
            !!dto.data.husnr &&
            !!dto.data.postnr &&
            !!dto.data.postnrnavn;
    }
    function _mapToAddressType(typeDto) {
        switch (typeDto) {
            case "adgangsadresse":
                return AddressType.Entrance;
            case "adresse":
                return AddressType.Address;
        }
    }
}
