export type Result<S> = R<S, Error, Cancellation>;
export declare function succeed<S>(success: S): R<S, never, never>;
export declare function fail<F>(failure: F): R<never, F, never>;
export declare function cancel<C>(cancellation: C): R<never, never, C>;
export declare function cancelStaleRequest(): R<never, never, Cancellation>;
export declare function mapSuccess<S, F, C, SS>(result: R<S, F, C>, successMapper: (success: S) => SS): R<SS, F, C>;
export declare function mapFailure<S, F, C, FF>(result: R<S, F, C>, failureMapper: (failure: F) => FF): R<S, FF, C>;
export declare function map<S, F, C, SS, FF, CC>(result: R<S, F, C>, successMapper: (success: S) => SS, failureMapper: (failure: F) => FF, cancellationMapper: (cancellation: C) => CC): R<SS, FF, CC>;
export type Cancellation = {
    readonly reason: string;
};
type R<S, F, C> = {
    type: "success";
    value: S;
} | {
    type: "failure";
    value: F;
} | {
    type: "cancellation";
    value: C;
};
export {};
